/*

Copyright (C) 2013  Francesco Poli <invernomuto@paranoici.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License with
the Debian GNU/Linux distribution in file /usr/share/common-licenses/GPL-2;
if not, write to the Free Software Foundation, Inc., 51 Franklin St,
Fifth Floor, Boston, MA 02110-1301, USA.

*/

.nocss { display: none; }

body
{
    color: black;
    background-color: white;
    padding: 0.1em;
}

/* links */
body a
{
    text-decoration: none;
}

body a:link
{
    color: blue;
    border-bottom: 1px blue solid;
}

body a:visited
{
    color: black;
    border-bottom: 1px black solid;
}

body a:hover
{
    color: blue;
    border-bottom: 1px blue dashed;
}

/* title */
div#page_title h1.title
{
    color: black;
    background-color: white;
    font-size: 180%;
    text-align: center;
    width: 80%;
    margin: 0.6em auto 0.1em;
    padding: 0.1em 0.1em 0.1em;
    border-top: 1px black solid;
    border-bottom: 1px black solid;
}

div#page_title h5.generated
{
    color: black;
    background-color: white;
    font-size: 80%;
    text-align: right;
    margin: 1.2em 0.1em 1.2em;
    padding: 0.1em 0.1em 0.1em;
}

/* tables */
div#page_body table.bug_table
{
    text-align: center;
    vertical-align: middle;
    margin: 1.5em auto 0.5em;
    border: 0px;
}

div#page_body table.bug_table .pkg
{
    text-align: right;
}

div#page_body table.bug_table .desc
{
    text-align: left;
}

div#page_body table.bug_table caption
{
    font-size: 110%;
    margin: 0.5em auto 0.5em;
}

div#page_body table.bug_table th
{
    margin: 1px;
    border: 0px;
    padding: 0.1em 0.5em;
}

div#page_body table.bug_table tr.critical  { background-color: #f99; }
div#page_body table.bug_table tr.grave     { background-color: #fc9; }
div#page_body table.bug_table tr.serious   { background-color: #fe9; }
div#page_body table.bug_table tr.important { background-color: #ccf; }
div#page_body table.bug_table tr.normal    { background-color: #cef; }
div#page_body table.bug_table tr.minor     { background-color: #af9; }
div#page_body table.bug_table tr.wishlist  { background-color: #eee; }

div#page_body table.bug_table td
{
    margin: 1px;
    border: 0px;
    padding: 0.1em 0.5em;
}
